@@ -1,11 +1,10 @@
 
 package AWS::S3;
 
-use Moose;
+use VSO;
 use Carp 'confess';
 use LWP::UserAgent::Determined;
 use HTTP::Response;
-use HTTP::Request::Common;
 use IO::Socket::INET;
 use Class::Load 'load_class';
 
@@ -13,97 +12,163 @@ use AWS::S3::ResponseParser;
 use AWS::S3::Owner;
 use AWS::S3::Bucket;
 
-our $VERSION = '0.11';
 
-has [qw/access_key_id secret_access_key/] => ( is => 'ro', isa => 'Str' );
+our $VERSION = '0.012';
 
-has 'secure' => (
-    is      => 'ro',
-    isa     => 'Bool',
-    lazy    => 1,
-    default => 0
+has 'access_key_id' => (
+  is    => 'ro'
 );
 
-has 'endpoint' => (
-    is      => 'ro',
-    isa     => 'Str',
-    lazy    => 1,
-    default => sub { 's3.amazonaws.com' },
+has 'secret_access_key' => (
+  is    => 'ro'
+);
+
+has 'secure' => (
+  is      => 'ro',
+  isa     => 'Int',
+  default => sub { 0 },
 );
 
 has 'ua' => (
-    is      => 'ro',
-    isa     => 'LWP::UserAgent',
-    default => sub { LWP::UserAgent::Determined->new }
+  is      => 'ro',
+  default => sub { LWP::UserAgent::Determined->new }
 );
 
-sub request {
-    my ( $s, $type, %args ) = @_;
 
-    my $class = "AWS::S3::Request::$type";
-    load_class( $class );
-    return $class->new( %args, s3 => $s, type => $type );
-}    # end request()
+sub request
+{
+  my ($s, $type, %args) = @_;
+  
+  my $class = "AWS::S3::Request::$type";
+  load_class($class);
+  return $class->new( s3 => $s, %args )->http_request;
+}# end request()
 
-sub owner {
-    my $s = shift;
 
-    my $type     = 'ListAllMyBuckets';
-    my $request  = $s->request( $type );
-    my $response = $request->request();
-    my $xpc      = $response->xpc;
-    return AWS::S3::Owner->new(
-        id           => $xpc->findvalue( '//s3:Owner/s3:ID' ),
-        display_name => $xpc->findvalue( '//s3:Owner/s3:DisplayName' ),
+sub owner
+{
+  my ($s) = @_;
+  
+  my $type = 'ListAllMyBuckets';
+  my $req = $s->request( $type,
+    bucket  => '',
+  );
+  
+  my $parser = AWS::S3::ResponseParser->new(
+    response        => $s->ua->request( $req ),
+    type            => $type,
+    expect_nothing  => 0,
+  );
+  
+  my $xpc = $parser->xpc;
+  
+  return AWS::S3::Owner->new(
+    id            => $xpc->findvalue('//s3:Owner/s3:ID'),
+    display_name  => $xpc->findvalue('//s3:Owner/s3:DisplayName'),
+  );
+}# end owner()
+
+
+sub buckets
+{
+  my ($s) = @_;
+  
+  my $type = 'ListAllMyBuckets';
+  my $req = $s->request( $type,
+    bucket  => '',
+  );
+  
+  my $parser = AWS::S3::ResponseParser->new(
+    response        => $s->ua->request( $req ),
+    type            => $type,
+    expect_nothing  => 0,
+  );
+  
+  my $xpc = $parser->xpc;
+  my @buckets = ( );
+  foreach my $node ( $xpc->findnodes('.//s3:Bucket') )
+  {
+    push @buckets, AWS::S3::Bucket->new(
+      name          => $xpc->findvalue('.//s3:Name', $node ),
+      creation_date => $xpc->findvalue('.//s3:CreationDate', $node),
+      s3            => $s,
     );
-}    # end owner()
+  }# end foreach()
+  
+  return @buckets;
+}# end buckets()
 
-sub buckets {
-    my ( $s ) = @_;
 
-    my $type     = 'ListAllMyBuckets';
-    my $request  = $s->request( $type );
-    my $response = $request->request();
+sub bucket
+{
+  my ($s, $name) = @_;
+  
+  my ($bucket) = grep { $_->name eq $name } $s->buckets
+    or return;
+  $bucket;
+}# end bucket()
 
-    my $xpc     = $response->xpc;
-    my @buckets = ();
-    foreach my $node ( $xpc->findnodes( './/s3:Bucket' ) ) {
-        push @buckets,
-          AWS::S3::Bucket->new(
-            name          => $xpc->findvalue( './/s3:Name',         $node ),
-            creation_date => $xpc->findvalue( './/s3:CreationDate', $node ),
-            s3            => $s,
-          );
-    }    # end foreach()
 
-    return @buckets;
-}    # end buckets()
+sub add_bucket
+{
+  my ($s, %args) = @_;
+  
+  my $type = 'CreateBucket';
+  my $req = $s->request( $type, bucket => $args{name} );
+  
+  my $parser = AWS::S3::ResponseParser->new(
+    response        => $s->_raw_response( $req ),
+    type            => $type,
+    expect_nothing  => 1,
+  );
+  
+  if( my $msg = $parser->friendly_error() )
+  {
+    die $msg;
+  }# end if()
+  
+  return $s->bucket( $args{name} );
+}# end add_bucket()
 
-sub bucket {
-    my ( $s, $name ) = @_;
 
-    my ( $bucket ) = grep { $_->name eq $name } $s->buckets
-      or return;
-    $bucket;
-}    # end bucket()
+sub _raw_response
+{
+  my ($s, $http_req) = @_;
+  
+  my ($host, $uri) = $http_req->uri =~ m{://(.+?)(/.?)$};
+  my $sock = IO::Socket::INET->new(
+    PeerAddr  => $host,
+    PeerPort  => 80, 
+    Proto     => 'tcp',
+  ) or die "Could not create socket: $!";
 
-sub add_bucket {
-    my ( $s, %args ) = @_;
+  my $req = <<"REQ";
+@{[ $http_req->method ]} $uri HTTP/1.1
+Host: $host
+Date: @{[ $http_req->header('Date') ]}
+Authorization: @{[ $http_req->header('Authorization') ]}
 
-    my $type     = 'CreateBucket';
-    my $request  = $s->request( $type, bucket => $args{name}, location => $args{location} );
-    my $response = $request->request();
 
-    if ( my $msg = $response->friendly_error() ) {
-        die $msg;
-    }    # end if()
+REQ
+  print $sock $req, $http_req->content;
 
-    return $s->bucket( $args{name} );
-}    # end add_bucket()
+  my @parts = ( );
+  while( <$sock> )
+  {
+    $_ =~ s{^\s+}{};
+    $_ =~ s{\s+$}{};
+    last if $_ eq '0';
+    push @parts, $_;
+    last unless length($_);
+  }# end while()
 
-__PACKAGE__->meta->make_immutable;
+  close($sock);
+  
+  return HTTP::Response->parse( join "\n", @parts );  
+}# end _raw_response()
 
-__END__
+
+1;# return true:
 
 =pod
 
@@ -111,56 +176,52 @@ __END__
 
 AWS::S3 - Lightweight interface to Amazon S3 (Simple Storage Service)
 
-=for html
-<a href='https://travis-ci.org/leejo/AWS-S3?branch=master'><img src='https://travis-ci.org/leejo/AWS-S3.svg?branch=master' /></a>
-<a href='https://coveralls.io/r/leejo/AWS-S3?branch=master'><img src='https://coveralls.io/repos/leejo/AWS-S3/badge.png?branch=master' alt='Coverage Status' /></a>
-
 =head1 SYNOPSIS
 
   use AWS::S3;
-
+  
   my $s3 = AWS::S3->new(
     access_key_id     => 'E654SAKIASDD64ERAF0O',
     secret_access_key => 'LgTZ25nCD+9LiCV6ujofudY1D6e2vfK0R4GLsI4H',
   );
-
+  
   # Add a bucket:
   my $bucket = $s3->add_bucket(
     name    => 'foo-bucket',
   );
-
+  
   # Set the acl:
   $bucket->acl( 'private' );
-
+  
   # Add a file:
   my $new_file = $bucket->add_file(
     key       => 'foo/bar.txt',
     contents  => \'This is the contents of the file',
   );
-
+  
   # You can also set the contents with a coderef:
   # Coderef should eturn a reference, not the actual string of content:
   $new_file = $bucket->add_file(
     key       => 'foo/bar.txt',
     contents  => sub { return \"This is the contents" }
   );
-
+  
   # Get the file:
   my $same_file = $bucket->file( 'foo/bar.txt' );
-
+  
   # Get the contents:
   my $scalar_ref = $same_file->contents;
   print $$scalar_ref;
-
+  
   # Update the contents with a scalar ref:
   $same_file->contents( \"New file contents" );
-
+  
   # Update the contents with a code ref:
   $same_file->contents( sub { return \"New file contents" } );
-
+  
   # Delete the file:
   $same_file->delete();
-
+  
   # Iterate through lots of files:
   my $iterator = $bucket->files(
     page_size   => 100,
@@ -177,17 +238,17 @@ AWS::S3 - Lightweight interface to Amazon S3 (Simple Storage Service)
       warn "\tContents: ", ${ $file->contents }, "\n";
     }# end foreach()
   }# end while()
-
+  
   # You can't delete a bucket until it's empty.
   # Empty a bucket like this:
   while( my @files = $iterator->next_page )
   {
     map { $_->delete } @files;
-
+    
     # Return to page 1:
     $iterator->page_number( 1 );
   }# end while()
-
+  
   # Now you can delete the bucket:
   $bucket->delete();
 
@@ -225,17 +286,11 @@ Optional.  Boolean.
 
 Default is C<0>
 
-=head2 endpoint
-
-Optional.  String.
-
-Default is C<s3.amazonaws.com>
-
 =head2 ua
 
 Optional.  Should be an instance of L<LWP::UserAgent> or a subclass of it.
 
-Defaults to creating a new instance of L<LWP::UserAgent::Determined>
+Defaults to creating a new instance of L<LWP::UserAgent>
 
 =head1 PUBLIC PROPERTIES
 
@@ -251,10 +306,6 @@ String.  Read-only.
 
 Boolean.  Read-only.
 
-=head2 endpoint
-
-String.  Read-only.
-
 =head2 ua
 
 L<LWP::UserAgent> object.  Read-only.
@@ -299,9 +350,9 @@ L<AWS::S3::Owner>
 
 =head1 AUTHOR
 
-Originally John Drago C<jdrago_999@yahoo.com>, currently maintained by Lee Johnson (LEEJO) C<leejo@cpan.org>
+John Drago <jdrago_999@yahoo.com>
 
-=head1 LICENSE
+=head1 LICENSE AND COPYRIGHT
 
 This software is Free software and may be used and redistributed under the same
 terms as any version of perl itself.
